@@ -2,6 +2,7 @@ use crate::{
     ConditionDifferentiableDistribution, ConditionedDistribution, DependentJoint, Distribution,
     DistributionError, Event, IndependentJoint, RandomVariable, ValueDifferentiableDistribution,
 };
+use opensrdk_linear_algebra::{Matrix, MatrixError, Vector};
 use rand::prelude::*;
 use std::{
     fmt::Debug,
@@ -17,7 +18,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     conditioned_distribution: ConditionedDistribution<D, T, U1, U2, F>,
     condition_diff: G,
@@ -31,7 +32,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     pub fn new(
         conditioned_distribution: ConditionedDistribution<D, T, U1, U2, F>,
@@ -53,7 +54,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(
@@ -72,7 +73,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     type Value = T;
     type Condition = U2;
@@ -108,7 +109,7 @@ where
     Rhs: Distribution<Value = TRhs, Condition = U2>,
     TRhs: RandomVariable,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     type Output = IndependentJoint<Self, Rhs, T, TRhs, U2>;
 
@@ -127,7 +128,7 @@ where
     Rhs: Distribution<Value = U2, Condition = URhs>,
     URhs: RandomVariable,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     type Output = DependentJoint<Self, Rhs, T, U2, URhs>;
 
@@ -144,7 +145,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     fn ln_diff_value(
         &self,
@@ -168,7 +169,7 @@ where
     U1: Event,
     U2: Event,
     F: Fn(&U2) -> Result<U1, DistributionError> + Clone + Send + Sync,
-    G: Fn(&U2) -> Result<Vec<f64>, DistributionError> + Clone + Send + Sync,
+    G: Fn(&U2) -> Matrix + Clone + Send + Sync,
 {
     fn ln_diff_condition(
         &self,
@@ -180,13 +181,11 @@ where
             .distribution
             .ln_diff_condition(x, &(self.conditioned_distribution.condition)(theta)?)
             .unwrap();
-        let g = &(self.condition_diff)(theta)?;
+        let g = &(self.condition_diff)(theta);
+
+        let diff_mat = f.row_mat() * g.t();
+        let diff = diff_mat.vec();
 
-        let diff = f
-            .iter()
-            .enumerate()
-            .map(|(i, fi)| fi * g[i])
-            .collect::<Vec<f64>>();
         Ok(diff)
     }
 }
@@ -215,7 +214,6 @@ mod tests {
           16.0, 17.0, 18.0, 19.0, 20.0, 21.0
         ))
         .unwrap();
-        println!("{:#?}", lsigma);
 
         let distr_prior = MultivariateNormal::new().condition(|theta: &Vec<f64>| {
             let f_mu = mu
@@ -226,12 +224,51 @@ mod tests {
             ExactMultivariateNormalParams::new(f_mu, PPTRF(lsigma.clone()))
         });
 
-        let g = |_theta: &Vec<f64>| Ok(vec![1.0, 1.0, 1.0, 1.0, 1.0, 1.0]);
+        let g = |_theta: &Vec<f64>| {
+            mat!(
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0
+            )
+        };
 
         let distr = ConditionDifferentiableConditionedDistribution::new(distr_prior, g);
 
         let x = distr
-            .sample(&vec![0.0, 0.0, 1.0, 2.0, 1.0, 2.0, 1.0, 2.0], &mut rng)
+            .sample(&vec![0.0, 1.0, 0.0, 1.0, 0.0, 1.0], &mut rng)
             .unwrap();
 
         println!("{:#?}", x);
@@ -247,7 +284,6 @@ mod tests {
            2.0,  3.0
         ))
         .unwrap();
-        println!("{:#?}", lsigma);
 
         let distr_prior = MultivariateNormal::new().condition(|theta: &Vec<f64>| {
             let f_mu = mu
@@ -258,7 +294,12 @@ mod tests {
             ExactMultivariateNormalParams::new(f_mu, PPTRF(lsigma.clone()))
         });
 
-        let g = |_theta: &Vec<f64>| Ok(vec![1.0, 1.0, 0.0, 0.0, 0.0, 0.0]);
+        let g = |_theta: &Vec<f64>| {
+            mat!(
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0;
+               1.0,  1.0,  1.0,  1.0,  1.0,  1.0
+            )
+        };
 
         let distr = ConditionDifferentiableConditionedDistribution::new(distr_prior, g);
 
