@@ -3,10 +3,12 @@ extern crate blas_src;
 #[cfg(test)]
 extern crate lapack_src;
 extern crate num_integer;
+extern crate opensrdk_kernel_method;
 pub extern crate opensrdk_linear_algebra;
 pub extern crate rand;
 extern crate rand_distr;
 extern crate rayon;
+extern crate special;
 extern crate thiserror;
 
 pub mod continuous;
@@ -14,7 +16,10 @@ pub mod discrete;
 pub mod distribution;
 pub mod mcmc;
 pub mod nonparametric;
+pub mod variational;
 
 pub use continuous::*;
 pub use discrete::*;
 pub use distribution::*;
+pub use mcmc::*;
+pub use variational::*;
